// query et traitement des contenus

import { defineStore } from 'pinia';
import REST from '../api/rest-axios';

import { findContentByPath } from '../utils/content/findContentByPath';
import { getCleanDate, fetchFromRelationships, getRelatedEtape } from '../utils/content/contentFetchUtils';
import { getCarteSensible, getTitreTexte, getChiffresCles, getDiaporama, getEntretien, getVideos } from '../utils/content/cleanParties';
import { getPartenaires, getGouvernance, getRessources } from '../utils/content/multiItemPages';

export const useContentStore = defineStore('content', {
    state: () => ({
        contentType: '',
        pageTitle: '',
        content: {},
        loading: false,
        error: null,
    }),
    actions: {
        async fetchContentData(path) {
            this.resetStore(false);
            const contentTypes = ['etape', 'static', 'gouvernance', 'partenaire', 'ressource'];
            try {
                const { contentType, rawContent } = await findContentByPath(contentTypes, path);
                this.contentType = contentType;

                if (this.contentType === 'etape' || this.contentType === 'static') {
                    const vignettePromise = fetchFromRelationships('field_vignette', rawContent.relationships);
                    const partiesPromise = fetchFromRelationships('field_parties', rawContent.relationships);

                    let previousEtapePromise, nextEtapePromise;

                    if (contentType === 'etape') {
                        // related étapes
                        previousEtapePromise = getRelatedEtape('previous', path);
                        nextEtapePromise = getRelatedEtape('next', path);

                        // coordinates
                        this.content.coordinates = {
                            lat: rawContent.attributes.field_geofield.lat,
                            lon: rawContent.attributes.field_geofield.lon,
                        };
                        // adresse
                        this.content.adresse = rawContent.attributes.field_adresse;
                        // étape number
                        this.content.etape_number = rawContent.attributes.field_arret_numero;
                        // couleur
                        this.content.couleur = rawContent.attributes.field_couleur;
                        // dates
                        this.content.dates = {
                            start: getCleanDate(rawContent.attributes.field_dates.value),
                            end: getCleanDate(rawContent.attributes.field_dates.end_value),
                        }
                    }

                    // pageTitle
                    this.pageTitle = rawContent.attributes.metatag.find(tag => tag.tag === "meta")?.attributes.content;

                    // contentTitle
                    this.content.contentTitle = rawContent.attributes.title;

                    const [vignetteData, partiesData] = await Promise.all([vignettePromise, partiesPromise]);

                    if (vignetteData) {
                        this.content.vignette = {
                            url: {
                                original: vignetteData.attributes.uri.url,
                                small: vignetteData.attributes.image_style_uri.content_small,
                                medium: vignetteData.attributes.image_style_uri.content_medium,
                                large: vignetteData.attributes.image_style_uri.content_large,
                            },
                            alt: rawContent.relationships.field_vignette.data.meta.alt
                        };
                    }

                    if (partiesData) {
                        const partiesPromises = partiesData.map(async (partie) => {
                            const partieType = partie.type.replace(/^paragraph--/, "");
                            let partieContent = { type: partieType };

                            switch (partieType) {
                                case 'carte_sensible':
                                    partieContent.carteSensible = await getCarteSensible(partie);
                                    break;
                                case 'titre_texte':
                                    const { titre, texte } = await getTitreTexte(partie);
                                    partieContent.titre = titre;
                                    partieContent.texte = texte;
                                    break;
                                case 'chiffres_cles':
                                    partieContent.chiffresCles = await getChiffresCles(partie);
                                    break;
                                case 'diaporama':
                                    partieContent.diaporama = await getDiaporama(partie);
                                    break;
                                case 'entretien':
                                    partieContent.entretien = await getEntretien(partie);
                                    break;
                                case 'exergue':
                                    partieContent.exergue = partie.attributes.field_texte_exergue.value;
                                    break;
                                case 'video':
                                    partieContent.videos = getVideos(partie);
                                    break;
                            }
                            return partieContent;
                        });

                        this.content.parties = await Promise.all(partiesPromises);
                    }

                    // related étapes
                    if (contentType === 'etape') {
                        const [prevContent, nextContent] = await Promise.all([previousEtapePromise, nextEtapePromise]);
                        this.content.previous = prevContent;
                        this.content.next = nextContent;
                    }
                } else {
                    // pages gouvernance (contact), ressources et partenaire
                    // ont plusieurs items par pages                    
                    const intro = await REST.get(`/jsonapi/config_pages/intro_${this.contentType}/`);
                    const introContent = intro.data.data[0];

                    this.pageTitle =
                        `${introContent.attributes.field_titre} ${introContent.attributes.metatag.find(tag => tag.tag === "meta")?.attributes.content}`;                    

                    this.content.contentTitle = introContent.attributes.field_titre;
                    this.content.intro = introContent.attributes.field_intro?.value;

                    let multiItemPageArray = [];                    

                    switch (this.contentType) {
                        case 'ressource':
                            multiItemPageArray = await getRessources(rawContent);
                            break;
                        case 'partenaire':
                            multiItemPageArray = await getPartenaires(rawContent);
                            break;
                        case 'gouvernance':
                            multiItemPageArray = await getGouvernance(rawContent);
                            break;
                    }

                    this.content[`${this.contentType}s`] = multiItemPageArray;
                    console.log(this.content);                    
                }
            } catch (error) {
                this.error = 'Failed to fetch data';
                console.error('Issue with getNodeData', error);
            } finally {
                this.loading = false;
            }
        },
        resetStore(forFrontDisplay) {
            this.contentType = '';
            this.pageTitle = '';
            this.content = {};
            this.loading = !forFrontDisplay;
            this.error = null;
        }
    },
});
